<?php
// Anonymous object with no __tostring method (not equal to anything).
echo "--- anonymous object without __tostring\n";
$obj = (object)array("a"=>"b");
var_dump($obj == "");     // not equal
var_dump($obj == "0");    // not equal
var_dump($obj == "1");    // not equal
var_dump($obj == "asdf"); // not equal

// Custom objects with __tostring method.
echo "--- custom objects with __tostring method\n";
class Test {
  function __tostring() {
    return "5";
  }
}

$test = new Test;
var_dump($test == "5");      // equal
var_dump($test == "+0.5e1"); // equal

echo "--- custom objects with __tostring method returning non-string\n";
set_error_handler(function($errno,$errstr){return true;});

class TestNonString {
  function __tostring() {
    return $this; // in case this returns non-string, it will 
                  // throw an error, but will return an
                  // empty string ("")
  }
}

var_dump(new TestNonString == ""); // bool(equal)

// This code would crash the PHP interpreter (stack exhaustion):
echo "--- recursive == operator in __tostring\n";
echo "skipping since it would ".
     "<a href=\"https://bugs.php.net/bug.php?id=46754\">crash the interpreter</a>\n";
//class Xyz {
//  function __tostring() {
//    //return $this->__tostring(); // -- this would not crash
//     return $this == "crash me"; // -- stack overflow crash
//   }
// }
// 
// new Xyx == "plz die";
//
// known: https://bugs.php.net/bug.php?id=46754

// PDORow using sqlite (has no cast_object, so it will be cast to LONG(1),
// so will be the string).
echo "--- PDORow object (has no cast_object handler)\n";
try {

  // Create a PDORow object using sqlite.
  $r = new \PDO("sqlite::memory:");
  $r = $r->query("SELECT sqlite_version()");
  $r = $r->fetch(\PDO::FETCH_LAZY);

  var_dump($r == "0");        // not equal
  var_dump($r == "");         // not equal
  var_dump($r == "anything"); // not equal

  var_dump($r == "1");               // equal
  var_dump($r == "  \t\r\n+0.01e2"); // equal

} catch (Exception $e) {
  echo "skipping PDORow test ({$e->message})\n";
}

// Note: This test does not cover a custom built-in cast_object supporting
// to STRING coversion. Some classess like Tidy do support it, but in the end
// the behavior is identical to that of the stdObjects (i.e. it returns a string
// if it's able to).

